% TERNAPIC Class provides interface between MATLAB scripts and
% LIBTAPIKERNEL.DLL and MEXCALLTERNAPI.MEXW32
classdef TernAPIC
    methods
        function obj = TernAPIC()
            mexCallTernAPI('init');
        end
        
        function setParameter(obj, name, value)
            if isnumeric(value)
                mexCallTernAPI('set_parameter', name, num2str(value));
            else
                mexCallTernAPI('set_parameter', name, value);
            end
        end
        
        function setParameters(obj, params)
            names = fieldnames(params);
            for i = 1:numel(names)
                name = names{i};
                value = params.(name);
                setParameter(obj, name, value);
            end
        end
        
        function setTdModel(obj, yaml)
            mexCallTernAPI('set_td_model', yaml);
        end
        
        function setDnum(obj, dnum)
            mexCallTernAPI('set_dnum', dnum);
        end
        
        function diag = calculateDiagram(obj)
            diag = PTDiagram(mexCallTernAPI('calculate_diagram'));
        end
        
        function setGibbsTriangleCfg(obj, yaml)
            mexCallTernAPI('set_gibbs_triangle_cfg', yaml);
        end
        
        function toGraph(obj, filename)
            mexCallTernAPI('to_graph', filename);
        end
        
        
        function loadRbFile(obj, filename)
            mexCallTernAPI('load_rb_file', filename);
        end
        
        function registerFunction(obj, name, handle)
            mexCallTernAPI('register_function', name, handle);
        end
        
        function unregisterFunctions(obj)
            mexCallTernAPI('unregister_functions');
        end
    end
end