#ifndef __TAPIGATE_H
#define __TAPIGATE_H

#ifdef __cplusplus
#define FEXTERN extern "C"
#else
#define FEXTERN extern
#endif

#define WMF_PTSECTION 0
#define WMF_POLYTSECTION 1

#define ELTIME_SURF 0
#define ELTIME_CONVHULL 1
#define ELTIME_REGIONS 2

#define POINTS_TIELINES 1
#define POINTS_EXTBOUND 2
#define POINTS_EXTBOUND_POLYGON 3
#define POINTS_TRILIST 4

#define RBVART_UNKNOWN 0
#define RBVART_MATRIX 1
#define RBVART_STRING 2
#define RBVART_HASH 3
#define RBVART_SCALAR 4

/* Ruby VM control functions */
FEXTERN char *tapi_errmsg();
FEXTERN int tapi_init_with_args(int argc, const char *argv[]);
FEXTERN int tapi_init();
FEXTERN int tapi_init_inruby();
FEXTERN int tapi_run_console();
FEXTERN void tapi_close();
FEXTERN int tapi_ispipe();
/* Work with thermodynamic model */
FEXTERN int tapi_calculate_diagram();
FEXTERN int tapi_set_td_model(const char *yaml_text);
FEXTERN int tapi_set_gibbs_triangle_cfg(const char *yaml_cfg);
FEXTERN int tapi_set_dnum(int dnum);
FEXTERN int tapi_set_parameter(const char *name, const char *value);
/* Work with polythermal diagram */
FEXTERN int tapi_calculate_polyt_diagram(const char *outfile, const char *yaml_cfg);
FEXTERN int tapi_polyt_diagram_resume();
FEXTERN int tapi_polyt_diagram_wait();
FEXTERN int tapi_polyt_diagram_isterminated(int *isterminated);
FEXTERN int tapi_polyt_diagram_get_T(double *T);
FEXTERN int tapi_polyt_diagram_stop();
/* Gibbs energy surface subroutines */
FEXTERN int tapi_get_num_of_points(int *num_of_points);
FEXTERN int tapi_get_xi(int index, double *xi);
FEXTERN int tapi_get_g(double *gi);
FEXTERN int tapi_get_ggrid_reg(int *reg);
/* Convex hull subroutines */
FEXTERN int tapi_get_num_of_facets(int *num_of_facets);
FEXTERN int tapi_get_kmat(int *kmat);
FEXTERN int tapi_get_convhull_reg(int *reg);
FEXTERN int tapi_get_convhull_phnum(unsigned char *reg);
/* Phase diagram regions subroutines */
FEXTERN int tapi_get_num_of_regions(int *num_of_regions);
FEXTERN int tapi_get_region_name(int index, char *name);
FEXTERN int tapi_get_region_phnum(int index, int *phnum);
FEXTERN int tapi_get_region_points(int index, int mode, int *buffer);
FEXTERN int tapi_get_region_num_of_points(int index, int mode, int *num_of_points);
/* Functions for work with QHULL log */
FEXTERN int tapi_get_qhull_log_length();
FEXTERN char *tapi_get_qhull_log_data();
FEXTERN void tapi_free_qhull_log();
/* Functions for work with WMF file buffer */
FEXTERN int tapi_get_wmf_length();
FEXTERN char *tapi_get_wmf_data();
/* Functions for interaction with license manager */
FEXTERN int tapi_licman_get_mac(char *mac_buffer);
FEXTERN int tapi_licman_get_expire_date(char *date_buffer);
FEXTERN int tapi_licman_get_username(char *username_buffer);
FEXTERN int tapi_licman_get_organization(char *organization_buffer);
FEXTERN int tapi_licman_is_readonly(int *is_readonly);
FEXTERN int tapi_licman_set_license(const char *lic);
/* Functions for working with polythermal sections */
FEXTERN int tapi_load_polythermal_data(const char *filename);
FEXTERN int tapi_free_polythermal_data();
FEXTERN int tapi_build_polyt_section_3(double x2_1, double x3_1, double x2_2, double x3_2);
FEXTERN int tapi_polyt_section_3_to_svg(const char *filename);
/* Functions for working with graphic objects */
FEXTERN int tapi_obj_handle_get(int graph_mode, int *obj_id);
FEXTERN int tapi_obj_handle_free(int obj_id);
FEXTERN int tapi_obj_handle_init_wmf(int obj_id);
FEXTERN int tapi_obj_handle_init_qhull_log(int obj_id);
FEXTERN int tapi_obj_handle_point_to_regname(int obj_id, double *xv, char *name);
FEXTERN int tapi_obj_handle_scr_point_to_regname(int obj_id, int *xv, char *name);
FEXTERN int tapi_obj_handle_scr_point_to_x23(int obj_id, int *xy, double *x23);
FEXTERN int tapi_obj_handle_scr_point_to_w23(int obj_id, int *xy, double *w23);
FEXTERN int tapi_obj_handle_scr_point_to_xT(int obj_id, int *xy, double *xT);
FEXTERN int tapi_obj_handle_to_graph(int obj_id, const char *filename);
FEXTERN int tapi_obj_handle_set_yaml_cfg(int obj_id, const char *yaml_cfg);
FEXTERN int tapi_obj_handle_get_elapsed_time(int obj_id, int time_id);
/* Functions for loading external scripts and external C functions */
FEXTERN int tapi_load_rb_file(const char *filename);
FEXTERN int tapi_rbvar_register_function(
	const char *name,
	void * (*f) (int, void **, void *));
FEXTERN void *tapi_rbvar_create_matrix(int m, int n);
FEXTERN void *tapi_rbvar_create_string(const char *str, int len);
FEXTERN void *tapi_rbvar_create_hash();
FEXTERN void *tapi_rbvar_get_nil();
FEXTERN char *tapi_rbvar_get_string(void *obj);
FEXTERN double tapi_rbvar_get_scalar(void *obj);
FEXTERN int tapi_rbvar_get_m(void *obj);
FEXTERN int tapi_rbvar_get_n(void *obj);
FEXTERN double *tapi_rbvar_get_matrix(void *obj);
FEXTERN int tapi_rbvar_get_type(void *obj);
FEXTERN int tapi_rbvar_set_field(void *obj, const char *fname, void *val);
FEXTERN void *tapi_rbvar_get_field(void *obj, const char *fname);
FEXTERN void *tapi_rbvar_get_field_name(void *obj, int index);
FEXTERN void tapi_rbvar_raise(const char *errmsg);
FEXTERN void tapi_rbvar_gc_disable();
FEXTERN void tapi_rbvar_gc_enable();
#endif
